/**
 * Button
 * v. 1.1
 * Copyright (C) 2015 Robert Ulbricht
 *
 * Button debounce and button repeat library. Typical debounce need
 * 1-2 ms. Library uses 25 ms as a default debounce time.
 * Default first repeat takes 500 ms. Rest repeats takes 100 ms. It
 * is similar to a typical keyboard in Windows.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */


#include "button.h"

///
/// Constructor
///
/// _pin - Pin connected to button
/// _callback - Callback routime from sketch
/// _debounce - Debounce time in ms
/// _repeat - Repeat time in ms. 0 - no repeat.
/// _pullup - Use internal pull-up resistor
///
Button::Button(uint8_t _pin, void (*_callback)(Button*), long _debounce, long _repeat, bool _pullup)
: pin(_pin), debounce(_debounce), repeat(_repeat)
{
callback=_callback;  
pinMode(pin, _pullup ? INPUT_PULLUP : INPUT);  
state=digitalRead(pin);
pressed = state==LOW;
}

///
/// Debounce button
///
void Button::Read()
{
unsigned long ms = millis();  
int value = digitalRead(pin);
if (value != last_value)
  last_debounce_time = ms;

if ((ms - last_debounce_time) > debounce)
  {
  if (value != state)
    {
    state = value;
    pressed = state == LOW;
    last_callback=ms;
    callback_first=true;
    callback(this);
    }
  }
last_value = value;

// handle repeat
if(pressed && repeat>0)
  {
  // first repeat = 5* repeat time
  unsigned long rp = callback_first ? 5*repeat : repeat;
  if((ms - last_callback) > rp)
    {
    last_callback=ms;
    callback_first=false;
    callback(this);
    }
  }
}

