/**
 * Button
 * v. 1.1
 * Copyright (C) 2015 Robert Ulbricht
 *
 * Button debounce and button repeat library. Typical debounce need
 * 1-2 ms. Library uses 25 ms as a default debounce time.
 * Default first repeat takes 500 ms. Rest repeats takes 100 ms. It
 * is similar to a typical keyboard in Windows.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef button_h
#define button_h

#include "Arduino.h"

///
/// Handling buttons
///
class Button
{
  uint8_t pin;
  long debounce;
  long repeat;
  int state;
  void (*callback)(Button*);
  int last_value;
  unsigned long last_debounce_time;
  unsigned long last_callback;
  bool callback_first; 
  
public:
  bool pressed;

  Button(uint8_t _pin, void (*_callback)(Button*), long _debounce=25, long _repeat=100, bool _pullup=false);
  void Read();
};

#endif
